@@ -644,7 +644,7 @@ const LoginDialog = new Lang.Class({
                            onComplete: function() {
                                Mainloop.idle_add(Lang.bind(this, function() {
                                    this._greeter.call_start_session_when_ready_sync(serviceName, true, null);
-                                   return false;
+                                   return GLib.SOURCE_REMOVE;
                                }));
                            },
                            onCompleteScope: this });
@@ -699,6 +699,7 @@ const LoginDialog = new Lang.Class({
                                                                      function() {
                                                                          this._timedLoginAnimationTime -= _TIMED_LOGIN_IDLE_THRESHOLD;
                                                                          hold.release();
+                                                                         return GLib.SOURCE_REMOVE;
                                                                      });
         return hold;
     },
@@ -763,7 +764,7 @@ const LoginDialog = new Lang.Class({
         global.stage.connect('captured-event',
                              Lang.bind(this, function(actor, event) {
                                 if (this._timedLoginDelay == undefined)
-                                    return false;
+                                    return Clutter.EVENT_PROPAGATE;
 
                                 if (event.type() == Clutter.EventType.KEY_PRESS ||
                                     event.type() == Clutter.EventType.BUTTON_PRESS) {
@@ -776,7 +777,7 @@ const LoginDialog = new Lang.Class({
                                     this._resetTimedLogin();
                                 }
 
-                                return false;
+                                return Clutter.EVENT_PROPAGATE;
                              }));
     },
 
@@ -885,6 +886,10 @@ const LoginDialog = new Lang.Class({
         Main.ctrlAltTabManager.removeGroup(this.dialogLayout);
     },
 
+    cancel: function() {
+        this._authPrompt.cancel();
+    },
+
     addCharacter: function(unichar) {
         this._authPrompt.addCharacter(unichar);
     },
