@@ -415,7 +415,7 @@ const AllView = new Lang.Class({
 
     _onScroll: function(actor, event) {
         if (this._displayingPopup)
-            return true;
+            return Clutter.EVENT_STOP;
 
         let direction = event.get_scroll_direction();
         if (direction == Clutter.ScrollDirection.UP)
@@ -423,7 +423,7 @@ const AllView = new Lang.Class({
         else if (direction == Clutter.ScrollDirection.DOWN)
             this.goToPage(this._currentPage + 1);
 
-        return true;
+        return Clutter.EVENT_STOP;
     },
 
     _onPan: function(action) {
@@ -454,17 +454,17 @@ const AllView = new Lang.Class({
 
     _onKeyPressEvent: function(actor, event) {
         if (this._displayingPopup)
-            return true;
+            return Clutter.EVENT_STOP;
 
         if (event.get_key_symbol() == Clutter.Page_Up) {
             this.goToPage(this._currentPage - 1);
-            return true;
+            return Clutter.EVENT_STOP;
         } else if (event.get_key_symbol() == Clutter.Page_Down) {
             this.goToPage(this._currentPage + 1);
-            return true;
+            return Clutter.EVENT_STOP;
         }
 
-        return false;
+        return Clutter.EVENT_PROPAGATE;
     },
 
     _getItemId: function(item) {
@@ -889,11 +889,24 @@ const AppSearchProvider = new Lang.Class({
     },
 
     getInitialResultSet: function(terms, callback, cancellable) {
-        callback(this._appSys.initial_search(terms));
+        let query = terms.join(' ');
+        let groups = Gio.DesktopAppInfo.search(query);
+        let usage = Shell.AppUsage.get_default();
+        let results = [];
+        groups.forEach(function(group) {
+            group = group.filter(function(appID) {
+                let app = Gio.DesktopAppInfo.new(appID);
+                return app && app.should_show();
+            });
+            results = results.concat(group.sort(function(a, b) {
+                return usage.compare('', a, b);
+            }));
+        });
+        callback(results);
     },
 
     getSubsearchResultSet: function(previousResults, terms, callback, cancellable) {
-        callback(this._appSys.subsearch(previousResults, terms));
+        this.getInitialResultSet(terms, callback, cancellable);
     },
 
     activateResult: function(result) {
@@ -1210,13 +1223,13 @@ const AppFolderPopup = new Lang.Class({
 
     _onKeyPress: function(actor, event) {
         if (!this._isOpen)
-            return false;
+            return Clutter.EVENT_PROPAGATE;
 
         if (event.get_key_symbol() != Clutter.KEY_Escape)
-            return false;
+            return Clutter.EVENT_PROPAGATE;
 
         this.popdown();
-        return true;
+        return Clutter.EVENT_STOP;
     },
 
     toggle: function() {
@@ -1357,13 +1370,13 @@ const AppIcon = new Lang.Class({
                 Lang.bind(this, function() {
                     this._menuTimeoutId = 0;
                     this.popupMenu();
-                    return false;
+                    return GLib.SOURCE_REMOVE;
                 }));
         } else if (button == 3) {
             this.popupMenu();
-            return true;
+            return Clutter.EVENT_STOP;
         }
-        return false;
+        return Clutter.EVENT_PROPAGATE;
     },
 
     _onClicked: function(actor, button) {
@@ -1375,7 +1388,6 @@ const AppIcon = new Lang.Class({
             // Last workspace is always empty
             let launchWorkspace = global.screen.get_workspace_by_index(global.screen.n_workspaces - 1);
             launchWorkspace.activate(global.get_current_time());
-            this.emit('launching');
             this.app.open_new_window(-1);
             Main.overview.hide();
         }
@@ -1434,7 +1446,6 @@ const AppIcon = new Lang.Class({
     },
 
     _onActivate: function (event) {
-        this.emit('launching');
         let modifiers = event.get_state();
 
         if (modifiers & Clutter.ModifierType.CONTROL_MASK
@@ -1503,7 +1514,9 @@ const AppIconMenu = new Lang.Class({
     _redisplay: function() {
         this.removeAll();
 
-        let windows = this._source.app.get_windows();
+        let windows = this._source.app.get_windows().filter(function(w) {
+            return Shell.WindowTracker.is_window_interesting(w);
+        });
 
         // Display the app windows menu items and the separator between windows
         // of the current desktop and other windows.
