@@ -50,11 +50,9 @@ const BackgroundCache = new Lang.Class({
                                         effects: Meta.BackgroundEffects.NONE });
 
         let content = null;
+
         let candidateContent = null;
         for (let i = 0; i < this._patterns.length; i++) {
-            if (!this._patterns[i])
-                continue;
-
             if (this._patterns[i].get_shading() != params.shadingType)
                 continue;
 
@@ -88,7 +86,6 @@ const BackgroundCache = new Lang.Class({
         }
 
         this._patterns.push(content);
-
         return content;
     },
 
@@ -116,9 +113,9 @@ const BackgroundCache = new Lang.Class({
 
     _removeContent: function(contentList, content) {
         let index = contentList.indexOf(content);
-
-        if (index >= 0)
-            contentList.splice(index, 1);
+        if (index < 0)
+            throw new Error("Trying to remove invalid content: " + content);
+        contentList.splice(index, 1);
     },
 
     removePatternContent: function(content) {
@@ -128,7 +125,8 @@ const BackgroundCache = new Lang.Class({
     removeImageContent: function(content) {
         let filename = content.get_filename();
 
-        if (filename && this._fileMonitors[filename])
+        let hasOtherUsers = this._images.some(function(content) { return filename == content.get_filename(); });
+        if (!hasOtherUsers)
             delete this._fileMonitors[filename];
 
         this._removeContent(this._images, content);
@@ -186,13 +184,17 @@ const BackgroundCache = new Lang.Class({
 
                                                   for (let j = 0; j < pendingLoad.callers.length; j++) {
                                                       if (pendingLoad.callers[j].onFinished) {
-                                                          if (content && pendingLoad.callers[j].shouldCopy) {
-                                                              content = object.copy(pendingLoad.callers[j].monitorIndex,
-                                                                                    pendingLoad.callers[j].effects);
+                                                          let newContent;
 
+                                                          if (content && pendingLoad.callers[j].shouldCopy) {
+                                                              newContent = content.copy(pendingLoad.callers[j].monitorIndex,
+                                                                                        pendingLoad.callers[j].effects);
+                                                              this._images.push(newContent);
+                                                          } else {
+                                                              newContent = content;
                                                           }
 
-                                                          pendingLoad.callers[j].onFinished(content);
+                                                          pendingLoad.callers[j].onFinished(newContent);
                                                       }
                                                   }
 
@@ -210,11 +212,9 @@ const BackgroundCache = new Lang.Class({
                                         onFinished: null });
 
         let content = null;
+
         let candidateContent = null;
         for (let i = 0; i < this._images.length; i++) {
-            if (!this._images[i])
-                continue;
-
             if (this._images[i].get_style() != params.style)
                 continue;
 
@@ -222,7 +222,7 @@ const BackgroundCache = new Lang.Class({
                 continue;
 
             if (params.style == GDesktopEnums.BackgroundStyle.SPANNED &&
-                this._images[i].monitor_index != this._monitorIndex)
+                this._images[i].monitor != params.monitorIndex)
                 continue;
 
             candidateContent = this._images[i];
@@ -262,6 +262,7 @@ const BackgroundCache = new Lang.Class({
             if (params.onLoaded) {
                 GLib.idle_add(GLib.PRIORITY_DEFAULT, Lang.bind(this, function() {
                     params.onLoaded(this._animation);
+                    return GLib.SOURCE_REMOVE;
                 }));
             }
         }
@@ -276,6 +277,7 @@ const BackgroundCache = new Lang.Class({
                            if (params.onLoaded) {
                                GLib.idle_add(GLib.PRIORITY_DEFAULT, Lang.bind(this, function() {
                                    params.onLoaded(this._animation);
+                                   return GLib.SOURCE_REMOVE;
                                }));
                            }
                        }));
@@ -375,7 +377,7 @@ const Background = new Lang.Class({
 
         GLib.idle_add(GLib.PRIORITY_DEFAULT, Lang.bind(this, function() {
             this.emit('loaded');
-            return false;
+            return GLib.SOURCE_REMOVE;
         }));
     },
 
@@ -414,27 +416,26 @@ const Background = new Lang.Class({
         this._fileWatches[filename] = signalId;
     },
 
-    _addImage: function(content, index, filename) {
-        content.brightness = this._brightness;
-        content.vignette_sharpness = this._vignetteSharpness;
+    _ensureImage: function(index) {
+        if (this._images[index])
+            return;
 
         let actor = new Meta.BackgroundActor();
-        actor.content = content;
 
         // The background pattern is the first actor in
         // the group, and all images should be above that.
         this.actor.insert_child_at_index(actor, index + 1);
-
         this._images[index] = actor;
-        this._watchCacheFile(filename);
     },
 
-    _updateImage: function(content, index, filename) {
+    _updateImage: function(index, content, filename) {
         content.brightness = this._brightness;
         content.vignette_sharpness = this._vignetteSharpness;
 
-        this._cache.removeImageContent(this._images[index].content);
-        this._images[index].content = content;
+        let image = this._images[index];
+        if (image.content)
+            this._cache.removeImageContent(content);
+        image.content = content;
         this._watchCacheFile(filename);
     },
 
@@ -482,11 +483,8 @@ const Background = new Lang.Class({
                                                   return;
                                               }
 
-                                              if (!this._images[i]) {
-                                                  this._addImage(content, i, files[i]);
-                                              } else {
-                                                  this._updateImage(content, i, files[i]);
-                                              }
+                                              this._ensureImage(i);
+                                              this._updateImage(i, content, files[i]);
 
                                               if (numPendingImages == 0) {
                                                   this._setLoaded();
@@ -521,7 +519,7 @@ const Background = new Lang.Class({
                                                       Lang.bind(this, function() {
                                                                     this._updateAnimationTimeoutId = 0;
                                                                     this._updateAnimation();
-                                                                    return false;
+                                                                    return GLib.SOURCE_REMOVE;
                                                                 }));
     },
 
@@ -541,24 +539,27 @@ const Background = new Lang.Class({
                                            });
     },
 
-    _loadFile: function(filename) {
+    _loadImage: function(filename) {
         this._cache.getImageContent({ monitorIndex: this._monitorIndex,
                                       effects: this._effects,
                                       style: this._style,
                                       filename: filename,
                                       cancellable: this._cancellable,
                                       onFinished: Lang.bind(this, function(content) {
-                                          if (!content) {
-                                              if (!this._cancellable.is_cancelled())
-                                                  this._loadAnimation(filename);
-                                              return;
+                                          if (content) {
+                                              this._ensureImage(0);
+                                              this._updateImage(0, content, filename);
                                           }
-
-                                          this._addImage(content, 0, filename);
                                           this._setLoaded();
                                       })
                                     });
+    },
 
+    _loadFile: function(filename) {
+        if (filename.endsWith('.xml'))
+            this._loadAnimation(filename);
+        else
+            this._loadImage(filename);
     },
 
     _load: function () {
@@ -638,7 +639,13 @@ const SystemBackground = new Lang.Class({
                                           this.emit('loaded');
                                       })
                                     });
-    }
+
+        this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
+    },
+
+    _onDestroy: function() {
+        this._cache.removeImageContent(this.actor.content);
+    },
 });
 Signals.addSignalMethods(SystemBackground.prototype);
 
@@ -726,8 +733,8 @@ const BackgroundManager = new Lang.Class({
         }
     },
 
-    _updateBackground: function(background, monitorIndex) {
-        let newBackground = this._createBackground(monitorIndex);
+    _updateBackground: function(background) {
+        let newBackground = this._createBackground();
         newBackground.vignetteSharpness = background.vignetteSharpness;
         newBackground.brightness = background.brightness;
         newBackground.visible = background.visible;
@@ -776,7 +783,7 @@ const BackgroundManager = new Lang.Class({
         background.changeSignalId = background.connect('changed', Lang.bind(this, function() {
             background.disconnect(background.changeSignalId);
             background.changeSignalId = 0;
-            this._updateBackground(background, this._monitorIndex);
+            this._updateBackground(background);
         }));
 
         background.actor.connect('destroy', Lang.bind(this, function() {
