@@ -69,7 +69,7 @@ const BoxPointer = new Lang.Class({
     _muteInput: function() {
         if (this._capturedEventId == 0)
             this._capturedEventId = this.actor.connect('captured-event',
-                                                       function() { return true; });
+                                                       function() { return Clutter.EVENT_STOP; });
     },
 
     _unmuteInput: function() {
@@ -121,6 +121,9 @@ const BoxPointer = new Lang.Class({
     },
 
     hide: function(animate, onComplete) {
+        if (!this.actor.visible)
+            return;
+
         let xOffset = 0;
         let yOffset = 0;
         let themeNode = this.actor.get_theme_node();
