@@ -249,7 +249,9 @@ const EndSessionDialog = new Lang.Class({
         this._subjectLabel = new St.Label({ style_class: 'end-session-dialog-subject' });
 
         messageLayout.add(this._subjectLabel,
-                          { y_fill:  false,
+                          { x_fill: false,
+                            y_fill:  false,
+                            x_align: St.Align.START,
                             y_align: St.Align.START });
 
         this._descriptionLabel = new St.Label({ style_class: 'end-session-dialog-description' });
@@ -409,7 +411,7 @@ const EndSessionDialog = new Lang.Class({
                 this._secondsLeft = this._totalSecondsToStayOpen - secondsElapsed;
                 if (this._secondsLeft > 0) {
                     this._sync();
-                    return true;
+                    return GLib.SOURCE_CONTINUE;
                 }
 
                 let dialogContent = DialogContent[this._type];
@@ -417,7 +419,7 @@ const EndSessionDialog = new Lang.Class({
                 this._confirm(button.signal);
                 this._timerId = 0;
 
-                return false;
+                return GLib.SOURCE_REMOVE;
             }));
     },
 
