@@ -413,15 +413,18 @@ const IconGrid = new Lang.Class({
     },
 
     removeAll: function() {
+        this._items = [];
+        this._grid.remove_all_children();
+    },
+
+    destroyAll: function() {
         this._items = [];
         this._grid.destroy_all_children();
     },
 
     addItem: function(item, index) {
-        if (!item.icon || !item.icon instanceof BaseIcon) {
-            log('Only items with a BaseIcon icon property can be added to IconGrid');
-            return;
-        }
+        if (!item.icon instanceof BaseIcon)
+            throw new Error('Only items with a BaseIcon icon property can be added to IconGrid');
 
         this._items.push(item);
         if (index !== undefined)
