@@ -1,6 +1,7 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
+const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
 const Meta = imports.gi.Meta;
 const Mainloop = imports.mainloop;
@@ -112,9 +113,6 @@ const Overview = new Lang.Class({
         // rendering options without duplicating the texture data.
         let monitor = Main.layoutManager.primaryMonitor;
 
-        this._desktopFade = new St.Bin();
-        Main.layoutManager.overviewGroup.add_child(this._desktopFade);
-
         let layout = new Clutter.BinLayout();
         this._stack = new Clutter.Actor({ layout_manager: layout });
         this._stack.add_constraint(new LayoutManager.MonitorConstraint({ primary: true }));
@@ -133,6 +131,9 @@ const Overview = new Lang.Class({
         Main.layoutManager.overviewGroup.add_child(this._backgroundGroup);
         this._bgManagers = [];
 
+        this._desktopFade = new St.Bin();
+        Main.layoutManager.overviewGroup.add_child(this._desktopFade);
+
         this._activationTime = 0;
 
         this.visible = false;           // animating to overview, in overview, animating out
@@ -147,7 +148,7 @@ const Overview = new Lang.Class({
         this._coverPane = new Clutter.Actor({ opacity: 0,
                                               reactive: true });
         Main.layoutManager.overviewGroup.add_child(this._coverPane);
-        this._coverPane.connect('event', Lang.bind(this, function (actor, event) { return true; }));
+        this._coverPane.connect('event', Lang.bind(this, function (actor, event) { return Clutter.EVENT_STOP; }));
 
         this._stack.add_actor(this._overview);
         Main.layoutManager.overviewGroup.add_child(this._stack);
@@ -369,7 +370,7 @@ const Overview = new Lang.Class({
                                                                     this._windowSwitchTimestamp);
                                                 this.hide();
                                                 this._lastHoveredWindow = null;
-                                                return false;
+                                                return GLib.SOURCE_REMOVE;
                                             }));
         }
 
@@ -378,6 +379,7 @@ const Overview = new Lang.Class({
 
     _onScrollEvent: function(actor, event) {
         this.emit('scroll-event', event);
+        return Clutter.EVENT_PROPAGATE;
     },
 
     addAction: function(action) {
