@@ -35,7 +35,8 @@ const RunDialog = new Lang.Class({
     Extends: ModalDialog.ModalDialog,
 
     _init : function() {
-        this.parent({ styleClass: 'run-dialog' });
+        this.parent({ styleClass: 'run-dialog',
+                      destroyOnClose: false });
 
         this._lockdownSettings = new Gio.Settings({ schema: LOCKDOWN_SCHEMA });
         this._terminalSettings = new Gio.Settings({ schema: TERMINAL_SCHEMA });
@@ -72,7 +73,9 @@ const RunDialog = new Lang.Class({
         let label = new St.Label({ style_class: 'run-dialog-label',
                                    text: _("Enter a Command") });
 
-        this.contentLayout.add(label, { y_align: St.Align.START });
+        this.contentLayout.add(label, { x_fill: false,
+                                        x_align: St.Align.START,
+                                        y_align: St.Align.START });
 
         let entry = new St.Entry({ style_class: 'run-dialog-entry',
                                    can_focus: true });
@@ -100,6 +103,8 @@ const RunDialog = new Lang.Class({
         this._errorMessage.clutter_text.line_wrap = true;
 
         this._errorBox.add(this._errorMessage, { expand: true,
+                                                 x_align: St.Align.START,
+                                                 x_fill: false,
                                                  y_align: St.Align.MIDDLE,
                                                  y_fill: false });
 
@@ -123,7 +128,7 @@ const RunDialog = new Lang.Class({
                     !this.pushModal())
                     this.close();
 
-                return true;
+                return Clutter.EVENT_STOP;
             }
             if (symbol == Clutter.Tab) {
                 let text = o.get_text();
@@ -137,9 +142,9 @@ const RunDialog = new Lang.Class({
                     o.insert_text(postfix, -1);
                     o.set_cursor_position(text.length + postfix.length);
                 }
-                return true;
+                return Clutter.EVENT_STOP;
             }
-            return false;
+            return Clutter.EVENT_PROPAGATE;
         }));
     },
 
