@@ -240,10 +240,6 @@ const NotificationsBox = new Lang.Class({
     },
 
     _sourceAdded: function(tray, source, initial) {
-        // Ignore transient sources
-        if (source.isTransient)
-            return;
-
         let obj = {
             visible: source.policy.showInLockScreen,
             detailed: source.policy.detailsInLockScreen,
@@ -673,11 +669,11 @@ const ScreenShield = new Lang.Class({
         // down after cancel.
 
         if (this._lockScreenState != MessageTray.State.SHOWN)
-            return false;
+            return Clutter.EVENT_PROPAGATE;
 
         let isEnter = (symbol == Clutter.KEY_Return || symbol == Clutter.KEY_KP_Enter);
         if (!isEnter && !(GLib.unichar_isprint(unichar) || symbol == Clutter.KEY_Escape))
-            return false;
+            return Clutter.EVENT_PROPAGATE;
 
         if (this._isLocked &&
             this._ensureUnlockDialog(true, true) &&
@@ -685,12 +681,12 @@ const ScreenShield = new Lang.Class({
             this._dialog.addCharacter(unichar);
 
         this._liftShield(true, 0);
-        return true;
+        return Clutter.EVENT_STOP;
     },
 
     _onLockScreenScroll: function(actor, event) {
         if (this._lockScreenState != MessageTray.State.SHOWN)
-            return false;
+            return Clutter.EVENT_PROPAGATE;
 
         let delta = 0;
         if (event.get_scroll_direction() == Clutter.ScrollDirection.UP)
@@ -705,7 +701,7 @@ const ScreenShield = new Lang.Class({
             this._liftShield(true, 0);
         }
 
-        return true;
+        return Clutter.EVENT_STOP;
     },
 
     _inhibitSuspend: function() {
@@ -756,7 +752,7 @@ const ScreenShield = new Lang.Class({
                              });
         }
 
-        return true;
+        return GLib.SOURCE_CONTINUE;
     },
 
     _onDragBegin: function() {
@@ -852,7 +848,7 @@ const ScreenShield = new Lang.Class({
                                                        Lang.bind(this, function() {
                                                            this._lockTimeoutId = 0;
                                                            this.lock(false);
-                                                           return false;
+                                                           return GLib.SOURCE_REMOVE;
                                                        }));
         }
 
@@ -1101,7 +1097,7 @@ const ScreenShield = new Lang.Class({
                 global.stage.disconnect(motionId);
             }
 
-            return false;
+            return Clutter.EVENT_PROPAGATE;
         }));
         this._cursorTracker.set_pointer_visible(false);
 
@@ -1114,6 +1110,7 @@ const ScreenShield = new Lang.Class({
 
             Mainloop.timeout_add(1000 * MANUAL_FADE_TIME, Lang.bind(this, function() {
                 this._activateFade(this._shortLightbox, MANUAL_FADE_TIME);
+                return GLib.SOURCE_REMOVE;
             }));
         } else {
             if (params.fadeToBlack)
