@@ -2,18 +2,20 @@
 
 const Clutter = imports.gi.Clutter;
 const Lang = imports.lang;
+const Gio = imports.gi.Gio;
 const Gtk = imports.gi.Gtk;
 const Meta = imports.gi.Meta;
 const Signals = imports.signals;
 const St = imports.gi.St;
 const Atk = imports.gi.Atk;
 
+const AppDisplay = imports.ui.appDisplay;
 const DND = imports.ui.dnd;
 const IconGrid = imports.ui.iconGrid;
 const Main = imports.ui.main;
 const Overview = imports.ui.overview;
+const RemoteSearch = imports.ui.remoteSearch;
 const Separator = imports.ui.separator;
-const Search = imports.ui.search;
 const Util = imports.misc.util;
 
 const SEARCH_PROVIDERS_SCHEMA = 'org.gnome.desktop.search-providers';
@@ -26,88 +28,95 @@ const SearchSystem = new Lang.Class({
 
     _init: function() {
         this._providers = [];
-        this._remoteProviders = [];
-        this.reset();
+
+        this._registerProvider(new AppDisplay.AppSearchProvider());
+
+        this._searchSettings = new Gio.Settings({ schema: SEARCH_PROVIDERS_SCHEMA });
+        this._searchSettings.connect('changed::disabled', Lang.bind(this, this._reloadRemoteProviders));
+        this._searchSettings.connect('changed::disable-external', Lang.bind(this, this._reloadRemoteProviders));
+        this._searchSettings.connect('changed::sort-order', Lang.bind(this, this._reloadRemoteProviders));
+
+        this._reloadRemoteProviders();
+
+        this._cancellable = new Gio.Cancellable();
     },
 
-    registerProvider: function (provider) {
-        provider.searchSystem = this;
+    addProvider: function(provider) {
         this._providers.push(provider);
-
-        if (provider.isRemoteProvider)
-            this._remoteProviders.push(provider);
+        this.emit('providers-changed');
     },
 
-    unregisterProvider: function (provider) {
-        let index = this._providers.indexOf(provider);
-        if (index == -1)
-            return;
-        provider.searchSystem = null;
-        this._providers.splice(index, 1);
+    _reloadRemoteProviders: function() {
+        let remoteProviders = this._providers.filter(function(provider) {
+            return provider.isRemoteProvider;
+        });
+        remoteProviders.forEach(Lang.bind(this, function(provider) {
+            this._unregisterProvider(provider);
+        }));
 
-        let remoteIndex = this._remoteProviders.indexOf(provider);
-        if (remoteIndex != -1)
-            this._remoteProviders.splice(remoteIndex, 1);
+        RemoteSearch.loadRemoteSearchProviders(Lang.bind(this, function(providers) {
+            providers.forEach(Lang.bind(this, this._registerProvider));
+        }));
+
+        this.emit('providers-changed');
+    },
+
+    _registerProvider: function (provider) {
+        this._providers.push(provider);
+    },
+
+    _unregisterProvider: function (provider) {
+        let index = this._providers.indexOf(provider);
+        this._providers.splice(index, 1);
     },
 
     getProviders: function() {
         return this._providers;
     },
 
-    getRemoteProviders: function() {
-        return this._remoteProviders;
-    },
-
     getTerms: function() {
-        return this._previousTerms;
+        return this._terms;
     },
 
     reset: function() {
-        this._previousTerms = [];
-        this._previousResults = [];
+        this._terms = [];
+        this._results = {};
     },
 
-    setResults: function(provider, results) {
-        let i = this._providers.indexOf(provider);
-        if (i == -1)
-            return;
-
-        this._previousResults[i] = [provider, results];
-        this.emit('search-updated', this._previousResults[i]);
+    _gotResults: function(results, provider) {
+        this._results[provider.id] = results;
+        this.emit('search-updated', provider, results);
     },
 
-    updateSearchResults: function(terms) {
+    setTerms: function(terms) {
+        this._cancellable.cancel();
+        this._cancellable.reset();
+
+        let previousResults = this._results;
+        let previousTerms = this._terms;
+        this.reset();
+
         if (!terms)
             return;
 
         let searchString = terms.join(' ');
-        let previousSearchString = this._previousTerms.join(' ');
+        let previousSearchString = previousTerms.join(' ');
         if (searchString == previousSearchString)
             return;
 
         let isSubSearch = false;
-        if (this._previousTerms.length > 0)
+        if (previousTerms.length > 0)
             isSubSearch = searchString.indexOf(previousSearchString) == 0;
 
-        let previousResultsArr = this._previousResults;
+        this._terms = terms;
 
-        let results = [];
-        this._previousTerms = terms;
-        this._previousResults = results;
-
-        if (isSubSearch) {
-            for (let i = 0; i < this._providers.length; i++) {
-                let [provider, previousResults] = previousResultsArr[i];
-                results.push([provider, []]);
-                provider.getSubsearchResultSet(previousResults, terms);
-            }
-        } else {
-            for (let i = 0; i < this._providers.length; i++) {
-                let provider = this._providers[i];
-                results.push([provider, []]);
-                provider.getInitialResultSet(terms);
-            }
-        }
+        this._providers.forEach(Lang.bind(this, function(provider) {
+            let previousProviderResults = previousResults[provider.id];
+            if (isSubSearch && previousProviderResults)
+                provider.getSubsearchResultSet(previousProviderResults, terms, Lang.bind(this, this._gotResults, provider), this._cancellable);
+            else
+                provider.getInitialResultSet(terms, Lang.bind(this, this._gotResults, provider), this._cancellable);
+        }));
     }
 });
 Signals.addSignalMethods(SearchSystem.prototype);
@@ -135,10 +144,9 @@ const MaxWidthBin = new Lang.Class({
 const SearchResult = new Lang.Class({
     Name: 'SearchResult',
 
-    _init: function(provider, metaInfo, terms) {
+    _init: function(provider, metaInfo) {
         this.provider = provider;
         this.metaInfo = metaInfo;
-        this.terms = terms;
 
         this.actor = new St.Button({ reactive: true,
                                      can_focus: true,
@@ -151,8 +159,7 @@ const SearchResult = new Lang.Class({
     },
 
     activate: function() {
-        this.provider.activateResult(this.metaInfo.id, this.terms);
-        Main.overview.toggle();
+        this.emit('activate', this.metaInfo.id);
     },
 
     setSelected: function(selected) {
@@ -162,6 +169,7 @@ const SearchResult = new Lang.Class({
             this.actor.remove_style_pseudo_class('selected');
     }
 });
+Signals.addSignalMethods(SearchResult.prototype);
 
 const ListSearchResult = new Lang.Class({
     Name: 'ListSearchResult',
@@ -169,8 +177,8 @@ const ListSearchResult = new Lang.Class({
 
     ICON_SIZE: 64,
 
-    _init: function(provider, metaInfo, terms) {
-        this.parent(provider, metaInfo, terms);
+    _init: function(provider, metaInfo) {
+        this.parent(provider, metaInfo);
 
         this.actor.style_class = 'list-search-result';
         this.actor.x_fill = true;
@@ -214,12 +222,12 @@ const GridSearchResult = new Lang.Class({
     Name: 'GridSearchResult',
     Extends: SearchResult,
 
-    _init: function(provider, metaInfo, terms) {
-        this.parent(provider, metaInfo, terms);
+    _init: function(provider, metaInfo) {
+        this.parent(provider, metaInfo);
 
         this.actor.style_class = 'grid-search-result';
 
-        let content = provider.createResultObject(metaInfo, terms);
+        let content = provider.createResultObject(metaInfo);
         let dragSource = null;
 
         if (content == null) {
@@ -292,6 +300,10 @@ const SearchResultsBase = new Lang.Class({
 
         let separator = new Separator.HorizontalSeparator({ style_class: 'search-section-separator' });
         this.actor.add(separator.actor);
+
+        this._resultDisplays = {};
+
+        this._cancellable = new Gio.Cancellable();
     },
 
     destroy: function() {
@@ -303,6 +315,7 @@ const SearchResultsBase = new Lang.Class({
     },
 
     clear: function() {
+        this._resultDisplays = {};
         this._clearResultDisplay();
         this.actor.hide();
     },
@@ -311,9 +324,38 @@ const SearchResultsBase = new Lang.Class({
         this.emit('key-focus-in', actor);
     },
 
+    _activateResult: function(result, id) {
+        this.provider.activateResult(id, this._terms);
+        Main.overview.toggle();
+    },
+
     _setMoreIconVisible: function(visible) {
     },
 
+    _ensureResultActors: function(results, callback) {
+        let metasNeeded = results.filter(Lang.bind(this, function(resultId) {
+            return this._resultDisplays[resultId] === undefined;
+        }));
+
+        if (metasNeeded.length === 0) {
+            callback();
+        } else {
+            this._cancellable.cancel();
+            this._cancellable.reset();
+
+            this.provider.getResultMetas(metasNeeded, Lang.bind(this, function(metas) {
+                metasNeeded.forEach(Lang.bind(this, function(resultId, i) {
+                    let meta = metas[i];
+                    let display = this._createResultDisplay(meta);
+                    display.connect('activate', Lang.bind(this, this._activateResult));
+                    display.actor.connect('key-focus-in', Lang.bind(this, this._keyFocusIn));
+                    this._resultDisplays[resultId] = display;
+                }));
+                callback();
+            }), this._cancellable);
+        }
+    },
+
     updateSearch: function(providerResults, terms, callback) {
         this._terms = terms;
 
@@ -326,15 +368,17 @@ const SearchResultsBase = new Lang.Class({
             let results = this.provider.filterResults(providerResults, maxResults);
             let hasMoreResults = results.length < providerResults.length;
 
-            this.provider.getResultMetas(results, Lang.bind(this, function(metas) {
-                this.clear();
+            this._ensureResultActors(results, Lang.bind(this, function() {
+                this._clearResultDisplay();
 
                 // To avoid CSS transitions causing flickering when
                 // the first search result stays the same, we hide the
                 // content while filling in the results.
                 this.actor.hide();
                 this._clearResultDisplay();
-                this._renderResults(metas);
+                results.forEach(Lang.bind(this, function(resultId) {
+                    this._addItem(this._resultDisplays[resultId]);
+                }));
                 this._setMoreIconVisible(hasMoreResults && this.provider.canLaunchSearch);
                 this.actor.show();
                 callback();
@@ -379,16 +423,16 @@ const ListSearchResults = new Lang.Class({
         return MAX_LIST_SEARCH_RESULTS_ROWS;
     },
 
-    _renderResults: function(metas) {
-        for (let i = 0; i < metas.length; i++) {
-            let display = new ListSearchResult(this.provider, metas[i], this._terms);
-            display.actor.connect('key-focus-in', Lang.bind(this, this._keyFocusIn));
-            this._content.add_actor(display.actor);
-        }
+    _clearResultDisplay: function () {
+        this._content.remove_all_children();
     },
 
-    _clearResultDisplay: function () {
-        this._content.destroy_all_children();
+    _createResultDisplay: function(meta) {
+        return new ListSearchResult(this.provider, meta);
+    },
+
+    _addItem: function(display) {
+        this._content.add_actor(display.actor);
     },
 
     getFirstResult: function() {
@@ -421,7 +465,8 @@ const GridSearchResults = new Lang.Class({
 
     _renderResults: function(metas) {
         for (let i = 0; i < metas.length; i++) {
-            let display = new GridSearchResult(this.provider, metas[i], this._terms);
+            let display = new GridSearchResult(this.provider, metas[i]);
+            display.connect('activate', Lang.bind(this, this._activateResult));
             display.actor.connect('key-focus-in', Lang.bind(this, this._keyFocusIn));
             this._grid.addItem(display);
         }
@@ -431,6 +476,14 @@ const GridSearchResults = new Lang.Class({
         this._grid.removeAll();
     },
 
+    _createResultDisplay: function(meta) {
+        return new GridSearchResult(this.provider, meta);
+    },
+
+    _addItem: function(display) {
+        this._grid.addItem(display);
+    },
+
     getFirstResult: function() {
         if (this._grid.visibleItemsCount() > 0)
             return this._grid.getItemAtIndex(0)._delegate;
@@ -443,10 +496,7 @@ Signals.addSignalMethods(GridSearchResults.prototype);
 const SearchResults = new Lang.Class({
     Name: 'SearchResults',
 
-    _init: function(searchSystem) {
-        this._searchSystem = searchSystem;
-        this._searchSystem.connect('search-updated', Lang.bind(this, this._updateResults));
-
+    _init: function() {
         this.actor = new St.BoxLayout({ name: 'searchResults',
                                         vertical: true });
 
@@ -481,14 +531,14 @@ const SearchResults = new Lang.Class({
                                        y_align: St.Align.MIDDLE });
         this._content.add(this._statusBin, { expand: true });
         this._statusBin.add_actor(this._statusText);
-        this._providers = this._searchSystem.getProviders();
-        this._providerDisplays = {};
-        for (let i = 0; i < this._providers.length; i++) {
-            this.createProviderDisplay(this._providers[i]);
-        }
 
         this._highlightDefault = false;
         this._defaultResult = null;
+
+        this._searchSystem = new SearchSystem();
+        this._searchSystem.connect('search-updated', Lang.bind(this, this._updateResults));
+        this._searchSystem.connect('providers-changed', Lang.bind(this, this._updateProviderDisplays));
+        this._updateProviderDisplays();
     },
 
     _onPan: function(action) {
@@ -502,31 +552,29 @@ const SearchResults = new Lang.Class({
         Util.ensureActorVisibleInScrollView(this._scrollView, actor);
     },
 
-    createProviderDisplay: function(provider) {
-        let providerDisplay = null;
+    _ensureProviderDisplay: function(provider) {
+        if (provider.display)
+            return;
 
-        if (provider.appInfo) {
+        let providerDisplay;
+        if (provider.appInfo)
             providerDisplay = new ListSearchResults(provider);
-        } else {
+        else
             providerDisplay = new GridSearchResults(provider);
-        }
 
         providerDisplay.connect('key-focus-in', Lang.bind(this, this._keyFocusIn));
-        this._providerDisplays[provider.id] = providerDisplay;
         this._content.add(providerDisplay.actor);
+        provider.display = providerDisplay;
     },
 
-    destroyProviderDisplay: function(provider) {
-        this._providerDisplays[provider.id].destroy();
-        delete this._providerDisplays[provider.id];
+    _updateProviderDisplays: function() {
+        this._searchSystem.getProviders().forEach(Lang.bind(this, this._ensureProviderDisplay));
     },
 
     _clearDisplay: function() {
-        for (let i = 0; i < this._providers.length; i++) {
-            let provider = this._providers[i];
-            let providerDisplay = this._providerDisplays[provider.id];
-            providerDisplay.clear();
-        }
+        this._searchSystem.getProviders().forEach(function(provider) {
+            provider.display.clear();
+        });
     },
 
     reset: function() {
@@ -542,12 +590,17 @@ const SearchResults = new Lang.Class({
         this._statusBin.show();
     },
 
+    setTerms: function(terms) {
+        this._searchSystem.setTerms(terms);
+    },
+
     _maybeSetInitialSelection: function() {
         let newDefaultResult = null;
 
-        for (let i = 0; i < this._providers.length; i++) {
-            let provider = this._providers[i];
-            let display = this._providerDisplays[provider.id];
+        let providers = this._searchSystem.getProviders();
+        for (let i = 0; i < providers.length; i++) {
+            let provider = providers[i];
+            let display = provider.display;
 
             if (!display.actor.visible)
                 continue;
@@ -570,16 +623,10 @@ const SearchResults = new Lang.Class({
     },
 
     _updateStatusText: function () {
-        let haveResults = false;
-
-        for (let i = 0; i < this._providers.length; i++) {
-            let provider = this._providers[i];
-            let display = this._providerDisplays[provider.id];
-            if (display.getFirstResult()) {
-                haveResults = true;
-                break;
-            }
-        }
+        let haveResults = this._searchSystem.getProviders().some(function(provider) {
+            let display = provider.display;
+            return (display.getFirstResult() != null);
+        });
 
         if (!haveResults) {
             this._statusText.set_text(_("No results."));
@@ -589,12 +636,11 @@ const SearchResults = new Lang.Class({
         }
     },
 
-    _updateResults: function(searchSystem, results) {
+    _updateResults: function(searchSystem, provider, results) {
         let terms = searchSystem.getTerms();
-        let [provider, providerResults] = results;
-        let display = this._providerDisplays[provider.id];
+        let display = provider.display;
 
-        display.updateSearch(providerResults, terms, Lang.bind(this, function() {
+        display.updateSearch(results, terms, Lang.bind(this, function() {
             this._maybeSetInitialSelection();
             this._updateStatusText();
         }));
