@@ -120,22 +120,22 @@ function _loadMode(file, info) {
 
     _modes[modeName] = {};
     let propBlacklist = ['unlockDialog'];
-    for (let prop in loadedData[DEFAULT_MODE]) {
+    for (let prop in _modes[DEFAULT_MODE]) {
         if (newMode[prop] !== undefined &&
             propBlacklist.indexOf(prop) == -1)
-            loadedData[modeName][prop] = newMode[prop];
+            _modes[modeName][prop] = newMode[prop];
     }
     _modes[modeName]['isPrimary'] = true;
 }
 
-function _getModes() {
+function _loadModes() {
     FileUtils.collectFromDatadirs('modes', false, _loadMode);
 }
 
 function listModes() {
-    let modes = _getModes();
-    modes.forEach(function() {
-        let names = Object.getOwnPropertyNames(modes);
+    _loadModes();
+    Mainloop.idle_add(function() {
+        let names = Object.getOwnPropertyNames(_modes);
         for (let i = 0; i < names.length; i++)
             if (_modes[names[i]].isPrimary)
                 print(names[i]);
@@ -148,6 +148,7 @@ const SessionMode = new Lang.Class({
     Name: 'SessionMode',
 
     _init: function() {
+        _loadModes();
         let isPrimary = (_modes[global.session_mode] &&
                          _modes[global.session_mode].isPrimary);
         let mode = isPrimary ? global.session_mode : 'user';
