@@ -1,6 +1,7 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
+const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
@@ -163,6 +164,7 @@ const SwitcherPopup = new Lang.Class({
                                                                Main.osdWindow.cancel();
                                                                this.actor.opacity = 255;
                                                                this._initialDelayTimeoutId = 0;
+                                                               return GLib.SOURCE_REMOVE;
                                                            }));
         return true;
     },
@@ -192,7 +194,7 @@ const SwitcherPopup = new Lang.Class({
         else
             this._keyPressHandler(keysym, backwards, action);
 
-        return true;
+        return Clutter.EVENT_STOP;
     },
 
     _keyReleaseEvent: function(actor, event) {
@@ -202,11 +204,12 @@ const SwitcherPopup = new Lang.Class({
         if (state == 0)
             this._finish(event.get_time());
 
-        return true;
+        return Clutter.EVENT_STOP;
     },
 
     _clickedOutside: function(actor, event) {
         this.destroy();
+        return Clutter.EVENT_PROPAGATE;
     },
 
     _scrollHandler: function(direction) {
@@ -218,6 +221,7 @@ const SwitcherPopup = new Lang.Class({
 
     _scrollEvent: function(actor, event) {
         this._scrollHandler(event.get_scroll_direction());
+        return Clutter.EVENT_PROPAGATE;
     },
 
     _itemActivatedHandler: function(n) {
@@ -251,6 +255,7 @@ const SwitcherPopup = new Lang.Class({
     _mouseTimedOut: function() {
         this._motionTimeoutId = 0;
         this.mouseActive = true;
+        return GLib.SOURCE_REMOVE;
     },
 
     _popModal: function() {
@@ -400,6 +405,7 @@ const SwitcherList = new Lang.Class({
 
     _onItemEnter: function (index) {
         this._itemEntered(index);
+        return Clutter.EVENT_PROPAGATE;
     },
 
     highlight: function(index, justOutline) {
@@ -509,7 +515,7 @@ const SwitcherList = new Lang.Class({
     _getPreferredWidth: function (actor, forHeight, alloc) {
         let [maxChildMin, maxChildNat] = this._maxChildWidth(forHeight);
 
-        let totalSpacing = Math.max(this._list.spacing * (this._items.length - 1), 0);
+        let totalSpacing = this._list.spacing * (this._items.length - 1);
         alloc.min_size = this._items.length * maxChildMin + totalSpacing;
         alloc.natural_size = alloc.min_size;
         this._minSize = alloc.min_size;
@@ -539,7 +545,7 @@ const SwitcherList = new Lang.Class({
         let childHeight = box.y2 - box.y1;
 
         let [maxChildMin, maxChildNat] = this._maxChildWidth(childHeight);
-        let totalSpacing = Math.max(this._list.spacing * (this._items.length - 1), 0);
+        let totalSpacing = this._list.spacing * (this._items.length - 1);
 
         let childWidth = Math.floor(Math.max(0, box.x2 - box.x1 - totalSpacing) / this._items.length);
 
