@@ -2,6 +2,7 @@
 
 const Clutter = imports.gi.Clutter;
 const Gio = imports.gi.Gio;
+const GLib = imports.gi.GLib;
 const Gtk = imports.gi.Gtk;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
@@ -157,17 +158,14 @@ const ViewSelector = new Lang.Class({
     },
 
     show: function() {
-        this._activePage = this._workspacesPage;
-
         this.reset();
-        this._appsPage.hide();
-        this._searchPage.hide();
+
         this._workspacesDisplay.show();
+        this._activePage = null;
+        this._showPage(this._workspacesPage);
 
         if (!this._workspacesDisplay.activeWorkspaceHasMaximizedWindows())
             Main.overview.fadeOutDesktop();
-
-        this._showPage(this._workspacesPage, true);
     },
 
     zoomFromOverview: function() {
@@ -189,6 +187,7 @@ const ViewSelector = new Lang.Class({
         params = Params.parse(params, { a11yFocus: null });
 
         let page = new St.Bin({ child: actor,
+                                visible: false,
                                 x_align: St.Align.START,
                                 y_align: St.Align.START,
                                 x_fill: true,
@@ -203,6 +202,7 @@ const ViewSelector = new Lang.Class({
                                                       this._a11yFocusPage(page);
                                                   })
                                             });;
+        page.hide();
         this.actor.add_actor(page);
         return page;
     },
@@ -212,7 +212,7 @@ const ViewSelector = new Lang.Class({
             oldPage.hide();
 
         this.emit('page-empty');
-
+        this._activePage.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
         this._activePage.show();
         Tweener.addTween(this._activePage,
             { opacity: 255,
@@ -268,7 +268,7 @@ const ViewSelector = new Lang.Class({
         // Ignore events while anything but the overview has
         // pushed a modal (system modals, looking glass, ...)
         if (Main.modalCount > 1)
-            return false;
+            return Clutter.EVENT_PROPAGATE;
 
         let modifiers = event.get_state();
         let symbol = event.get_key_symbol();
@@ -280,19 +280,11 @@ const ViewSelector = new Lang.Class({
                 this._showAppsButton.checked = false;
             else
                 Main.overview.hide();
-            return true;
+            return Clutter.EVENT_STOP;
         } else if (this._shouldTriggerSearch(symbol)) {
             this.startSearch(event);
-        } else if (!this._searchActive) {
-            if (symbol == Clutter.Tab || symbol == Clutter.Down) {
-                this._activePage.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
-                return true;
-            } else if (symbol == Clutter.ISO_Left_Tab) {
-                this._activePage.navigate_focus(null, Gtk.DirectionType.TAB_BACKWARD, false);
-                return true;
-            }
         }
-        return false;
+        return Clutter.EVENT_PROPAGATE;
     },
 
     _searchCancelled: function() {
@@ -408,7 +400,7 @@ const ViewSelector = new Lang.Class({
         if (symbol == Clutter.Escape) {
             if (this._isActivated()) {
                 this.reset();
-                return true;
+                return Clutter.EVENT_STOP;
             }
         } else if (this._searchActive) {
             let arrowNext, nextDirection;
@@ -422,18 +414,18 @@ const ViewSelector = new Lang.Class({
 
             if (symbol == Clutter.Tab) {
                 this._searchResults.navigateFocus(Gtk.DirectionType.TAB_FORWARD);
-                return true;
+                return Clutter.EVENT_STOP;
             } else if (symbol == Clutter.ISO_Left_Tab) {
                 this._focusTrap.can_focus = false;
                 this._searchResults.navigateFocus(Gtk.DirectionType.TAB_BACKWARD);
                 this._focusTrap.can_focus = true;
-                return true;
+                return Clutter.EVENT_STOP;
             } else if (symbol == Clutter.Down) {
                 this._searchResults.navigateFocus(Gtk.DirectionType.DOWN);
-                return true;
+                return Clutter.EVENT_STOP;
             } else if (symbol == arrowNext && this._text.position == -1) {
                 this._searchResults.navigateFocus(nextDirection);
-                return true;
+                return Clutter.EVENT_STOP;
             } else if (symbol == Clutter.Return || symbol == Clutter.KP_Enter) {
                 // We can't connect to 'activate' here because search providers
                 // might want to do something with the modifiers in activateDefault.
@@ -442,10 +434,10 @@ const ViewSelector = new Lang.Class({
                     this._doSearch();
                 }
                 this._searchResults.activateDefault();
-                return true;
+                return Clutter.EVENT_STOP;
             }
         }
-        return false;
+        return Clutter.EVENT_PROPAGATE;
     },
 
     _onCapturedEvent: function(actor, event) {
@@ -460,7 +452,7 @@ const ViewSelector = new Lang.Class({
             }
         }
 
-        return false;
+        return Clutter.EVENT_PROPAGATE;
     },
 
     _doSearch: function () {
@@ -470,6 +462,8 @@ const ViewSelector = new Lang.Class({
 
         this._searchResults.setTerms(terms);
         this._showPage(this._searchPage);
+
+        return GLib.SOURCE_REMOVE;
     },
 
     getActivePage: function() {
