@@ -1,6 +1,7 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
 
 const Clutter = imports.gi.Clutter;
+const GLib = imports.gi.GLib;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
@@ -63,11 +64,12 @@ const WindowClone = new Lang.Class({
         // the invisible border; this is inconvenient; rather than trying
         // to compensate all over the place we insert a ClutterActor into
         // the hierarchy that is sized to only the visible portion.
-        this.actor = new Clutter.Actor({ reactive: true,
-                                         x: this.origX,
-                                         y: this.origY,
-                                         width: outerRect.width,
-                                         height: outerRect.height });
+        this.actor = new St.Widget({ reactive: true,
+                                     can_focus: true,
+                                     x: this.origX,
+                                     y: this.origY,
+                                     width: outerRect.width,
+                                     height: outerRect.height });
 
         this.actor.add_child(this._windowClone);
 
@@ -85,10 +87,10 @@ const WindowClone = new Lang.Class({
         let clickAction = new Clutter.ClickAction();
         clickAction.connect('clicked', Lang.bind(this, this._onClicked));
         clickAction.connect('long-press', Lang.bind(this, this._onLongPress));
-
         this.actor.add_action(clickAction);
-
         this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
+        this.actor.connect('key-press-event', Lang.bind(this, this._onKeyPress));
+        this.actor.connect('enter-event', Lang.bind(this, this._onEnter));
 
         this._draggable = DND.makeDraggable(this.actor,
                                             { restoreOnSuccess: true,
@@ -197,11 +199,30 @@ const WindowClone = new Lang.Class({
         this.disconnectAll();
     },
 
-    _onClicked: function(action, actor) {
+    _activate: function() {
         this._selected = true;
         this.emit('selected', global.get_current_time());
     },
 
+    _onKeyPress: function(actor, event) {
+        let symbol = event.get_key_symbol();
+        let isEnter = (symbol == Clutter.KEY_Return || symbol == Clutter.KEY_KP_Enter);
+        if (isEnter) {
+            this._activate();
+            return true;
+        }
+
+        return false;
+    },
+
+    _onEnter: function() {
+        this.actor.grab_key_focus();
+    },
+
+    _onClicked: function(action, actor) {
+        this._activate();
+    },
+
     _onLongPress: function(action, actor, state) {
         // Take advantage of the Clutter policy to consider
         // a long-press canceled when the pointer movement
@@ -301,6 +322,10 @@ const WindowOverlay = new Lang.Class({
                                   Lang.bind(this, this._onEnter));
         windowClone.actor.connect('leave-event',
                                   Lang.bind(this, this._onLeave));
+        windowClone.actor.connect('key-focus-in',
+                                  Lang.bind(this, this._onEnter));
+        windowClone.actor.connect('key-focus-out',
+                                  Lang.bind(this, this._onLeave));
 
         this._windowAddedId = 0;
 
@@ -448,7 +473,7 @@ const WindowOverlay = new Lang.Class({
             Mainloop.idle_add(Lang.bind(this,
                                         function() {
                                             this._windowClone.emit('selected');
-                                            return false;
+                                            return GLib.SOURCE_REMOVE;
                                         }));
         }
     },
@@ -512,15 +537,17 @@ const WindowOverlay = new Lang.Class({
         // as the close button will be shown as needed when the overlays
         // are shown again
         if (this._hidden)
-            return;
+            return Clutter.EVENT_PROPAGATE;
 
         this._animateVisible();
         this.emit('show-close-button');
+        return Clutter.EVENT_PROPAGATE;
     },
 
     _onLeave: function() {
         if (this._idleToggleCloseId == 0)
             this._idleToggleCloseId = Mainloop.timeout_add(750, Lang.bind(this, this._idleToggleCloseButton));
+        return Clutter.EVENT_PROPAGATE;
     },
 
     _idleToggleCloseButton: function() {
@@ -530,7 +557,7 @@ const WindowOverlay = new Lang.Class({
             !this.closeButton.has_pointer)
             this._animateInvisible();
 
-        return false;
+        return GLib.SOURCE_REMOVE;
     },
 
     hideCloseButton: function() {
@@ -1198,18 +1225,18 @@ const Workspace = new Lang.Class({
             // store current cursor position
             this._cursorX = x;
             this._cursorY = y;
-            return true;
+            return GLib.SOURCE_CONTINUE;
         }
 
         let actorUnderPointer = global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, x, y);
         for (let i = 0; i < this._windows.length; i++) {
             if (this._windows[i].actor == actorUnderPointer)
-                return true;
+                return GLib.SOURCE_CONTINUE;
         }
 
         this._recalculateWindowPositions(WindowPositionFlags.ANIMATE);
         this._repositionWindowsId = 0;
-        return false;
+        return GLib.SOURCE_REMOVE;
     },
 
     _doRemoveWindow : function(metaWin) {
@@ -1284,7 +1311,7 @@ const Workspace = new Lang.Class({
                                                 metaWin.get_compositor_private() &&
                                                 metaWin.get_workspace() == this.metaWorkspace)
                                                 this._doAddWindow(metaWin);
-                                            return false;
+                                            return GLib.SOURCE_REMOVE;
                                         }));
             return;
         }
