@@ -2,6 +2,7 @@
 
 const Clutter = imports.gi.Clutter;
 const Gio = imports.gi.Gio;
+const GLib = imports.gi.GLib;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
@@ -130,7 +131,7 @@ const WindowClone = new Lang.Class({
     _onButtonRelease : function (actor, event) {
         this.emit('selected', event.get_time());
 
-        return true;
+        return Clutter.EVENT_STOP;
     },
 
     _onDragBegin : function (draggable, time) {
@@ -325,7 +326,7 @@ const WorkspaceThumbnail = new Lang.Class({
                                                 metaWin.get_compositor_private() &&
                                                 metaWin.get_workspace() == this.metaWorkspace)
                                                 this._doAddWindow(metaWin);
-                                            return false;
+                                            return GLib.SOURCE_REMOVE;
                                         }));
             return;
         }
@@ -561,7 +562,7 @@ const ThumbnailsBox = new Lang.Class({
 
         this._thumbnails = [];
 
-        this.actor.connect('button-press-event', function() { return true; });
+        this.actor.connect('button-press-event', function() { return Clutter.EVENT_STOP; });
         this.actor.connect('button-release-event', Lang.bind(this, this._onButtonRelease));
 
         Main.overview.connect('showing',
@@ -606,7 +607,7 @@ const ThumbnailsBox = new Lang.Class({
             }
         }
 
-        return true;
+        return Clutter.EVENT_STOP;
     },
 
     _onDragBegin: function() {
