@@ -30,6 +30,7 @@ const WorkspacesViewBase = new Lang.Class({
         this.actor = new St.Widget({ style_class: 'workspaces-view',
                                      reactive: true });
         this.actor.connect('destroy', Lang.bind(this, this._onDestroy));
+        global.focus_manager.add_group(this.actor);
 
         // The actor itself isn't a drop target, so we don't want to pick on its area
         this.actor.set_size(0, 0);
@@ -371,11 +372,21 @@ const ExtraWorkspaceView = new Lang.Class({
     },
 });
 
+const DelegateFocusNavigator = new Lang.Class({
+    Name: 'DelegateFocusNavigator',
+    Extends: St.Widget,
+
+    vfunc_navigate_focus: function(from, direction) {
+        return this._delegate.navigateFocus(from, direction);
+    },
+});
+
 const WorkspacesDisplay = new Lang.Class({
     Name: 'WorkspacesDisplay',
 
     _init: function() {
-        this.actor = new St.Widget({ clip_to_allocation: true });
+        this.actor = new DelegateFocusNavigator({ clip_to_allocation: true });
+        this.actor._delegate = this;
         this.actor.connect('notify::allocation', Lang.bind(this, this._updateWorkspacesActualGeometry));
         this.actor.connect('parent-set', Lang.bind(this, this._parentSet));
 
@@ -437,6 +448,10 @@ const WorkspacesDisplay = new Lang.Class({
         return false;
     },
 
+    navigateFocus: function(from, direction) {
+        return this._getPrimaryView().actor.navigate_focus(from, direction, false);
+    },
+
     show: function() {
         this._updateWorkspacesViews();
         for (let i = 0; i < this._workspacesViews.length; i++)
@@ -599,7 +614,7 @@ const WorkspacesDisplay = new Lang.Class({
 
     _onScrollEvent: function(actor, event) {
         if (!this.actor.mapped)
-            return false;
+            return Clutter.EVENT_PROPAGATE;
         let activeWs = global.screen.get_active_workspace();
         let ws;
         switch (event.get_scroll_direction()) {
@@ -610,10 +625,10 @@ const WorkspacesDisplay = new Lang.Class({
             ws = activeWs.get_neighbor(Meta.MotionDirection.DOWN);
             break;
         default:
-            return false;
+            return Clutter.EVENT_PROPAGATE;
         }
         Main.wm.actionMoveWorkspace(ws);
-        return true;
+        return Clutter.EVENT_STOP;
     }
 });
 Signals.addSignalMethods(WorkspacesDisplay.prototype);
