@@ -69,10 +69,6 @@ typedef struct {
   guint  last_gc_seconds_ago;
 } ShellMemoryInfo;
 
-void     shell_global_get_memory_info      (ShellGlobal     *global,
-                                            ShellMemoryInfo *meminfo);
-
-
 /* Run-at-leisure API */
 void shell_global_begin_work     (ShellGlobal          *global);
 void shell_global_end_work       (ShellGlobal          *global);
@@ -129,13 +125,19 @@ void     shell_global_reexec_self               (ShellGlobal  *global);
 
 const char *     shell_global_get_session_mode  (ShellGlobal  *global);
 
-void     shell_global_set_runtime_state      (ShellGlobal  *global,
-                                              const char   *property_name,
-                                              GVariant     *variant);
+void     shell_global_set_runtime_state         (ShellGlobal  *global,
+                                                 const char   *property_name,
+                                                 GVariant     *variant);
 GVariant * shell_global_get_runtime_state       (ShellGlobal  *global,
                                                  const char   *property_type,
                                                  const char   *property_name);
 
+void     shell_global_set_persistent_state      (ShellGlobal  *global,
+                                                 const char   *property_name,
+                                                 GVariant     *variant);
+GVariant * shell_global_get_persistent_state    (ShellGlobal  *global,
+                                                 const char   *property_type,
+                                                 const char   *property_name);
 
 G_END_DECLS
 
