@@ -14,11 +14,14 @@
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
 
+#include <cogl/cogl.h>
+#include <meta/screen.h>
+#include <meta/meta-cursor-tracker.h>
+
 #include "shell-recorder-src.h"
 #include "shell-recorder.h"
 
 #include <clutter/x11/clutter-x11.h>
-#include <X11/extensions/Xfixes.h>
 #include <X11/extensions/XInput.h>
 #include <X11/extensions/XInput2.h>
 
@@ -64,21 +67,16 @@ struct _ShellRecorder {
   int pointer_x;
   int pointer_y;
 
-  gboolean have_xfixes;
-  int xfixes_event_base;
-
   int xinput_opcode;
 
-  CoglHandle recording_icon; /* icon shown while playing */
-
   GSettings *a11y_settings;
   gboolean draw_cursor;
+  MetaCursorTracker *cursor_tracker;
   cairo_surface_t *cursor_image;
+  guint8 *cursor_memory;
   int cursor_hot_x;
   int cursor_hot_y;
 
-  gboolean only_paint; /* Used to temporarily suppress recording */
-
   int framerate;
   char *pipeline_description;
   char *file_template;
@@ -125,6 +123,7 @@ static void recorder_pipeline_closed   (RecorderPipeline *pipeline);
 
 enum {
   PROP_0,
+  PROP_SCREEN,
   PROP_STAGE,
   PROP_FRAMERATE,
   PROP_PIPELINE,
@@ -170,57 +169,6 @@ G_DEFINE_TYPE(ShellRecorder, shell_recorder, G_TYPE_OBJECT);
  */
 #define DEFAULT_MEMORY_TARGET (512*1024)
 
-/* Create an emblem to show at the lower-left corner of the stage while
- * recording. The emblem is drawn *after* we record the frame so doesn't
- * show up in the frame.
- */
-static CoglHandle
-create_recording_icon (void)
-{
-  cairo_surface_t *surface = cairo_image_surface_create (CAIRO_FORMAT_ARGB32, 32, 32);
-  cairo_t *cr;
-  cairo_pattern_t *pat;
-  CoglHandle texture;
-
-  cr = cairo_create (surface);
-
-  /* clear to transparent */
-  cairo_save (cr);
-  cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
-  cairo_paint (cr);
-  cairo_restore (cr);
-
-  /* radial "glow" */
-  pat = cairo_pattern_create_radial (16, 16, 6,
-                                     16, 16, 14);
-  cairo_pattern_add_color_stop_rgba (pat, 0.0,
-                                     1, 0, 0, 1); /* opaque red */
-  cairo_pattern_add_color_stop_rgba (pat, 1.0,
-                                     1, 0, 0, 0); /* transparent red */
-
-  cairo_set_source (cr, pat);
-  cairo_paint (cr);
-  cairo_pattern_destroy (pat);
-
-  /* red circle */
-  cairo_arc (cr, 16, 16, 8,
-             0, 2 * M_PI);
-  cairo_set_source_rgb (cr, 1, 0, 0);
-  cairo_fill (cr);
-
-  cairo_destroy (cr);
-
-  texture = cogl_texture_new_from_data (32, 32,
-                                        COGL_TEXTURE_NONE,
-                                        CLUTTER_CAIRO_FORMAT_ARGB32,
-                                        COGL_PIXEL_FORMAT_ANY,
-                                        cairo_image_surface_get_stride (surface),
-                                        cairo_image_surface_get_data (surface));
-  cairo_surface_destroy (surface);
-
-  return texture;
-}
-
 static guint
 get_memory_target (void)
 {
@@ -278,7 +226,6 @@ shell_recorder_init (ShellRecorder *recorder)
 
   recorder->gdk_screen = gdk_screen_get_default ();
 
-  recorder->recording_icon = create_recording_icon ();
   recorder->memory_target = get_memory_target();
 
   recorder->a11y_settings = g_settings_new (A11Y_APPS_SCHEMA);
@@ -298,13 +245,13 @@ shell_recorder_finalize (GObject  *object)
 
   if (recorder->cursor_image)
     cairo_surface_destroy (recorder->cursor_image);
+  if (recorder->cursor_memory)
+    g_free (recorder->cursor_memory);
 
   recorder_set_stage (recorder, NULL);
   recorder_set_pipeline (recorder, NULL);
   recorder_set_file_template (recorder, NULL);
 
-  cogl_handle_unref (recorder->recording_icon);
-
   g_clear_object (&recorder->a11y_settings);
 
   G_OBJECT_CLASS (shell_recorder_parent_class)->finalize (object);
@@ -340,20 +287,7 @@ recorder_update_memory_used (ShellRecorder *recorder,
     }
 
   if (memory_used != recorder->memory_used)
-    {
-      recorder->memory_used = memory_used;
-      if (repaint)
-        {
-          /* In other cases we just queue a redraw even if we only need
-           * to repaint and not redraw a frame, but having changes in
-           * memory usage cause frames to be painted and memory used
-           * seems like a bad idea.
-           */
-          recorder->only_paint = TRUE;
-          clutter_stage_ensure_redraw (recorder->stage);
-          recorder->only_paint = FALSE;
-        }
-    }
+    recorder->memory_used = memory_used;
 }
 
 /* Timeout used to avoid not drawing for more than MAXIMUM_PAUSE_TIME
@@ -393,38 +327,24 @@ recorder_remove_redraw_timeout (ShellRecorder *recorder)
 static void
 recorder_fetch_cursor_image (ShellRecorder *recorder)
 {
-  XFixesCursorImage *cursor_image;
-  guchar *data;
+  CoglTexture *texture;
+  int width, height;
   int stride;
-  int i, j;
+  guint8 *data;
 
-  if (!recorder->have_xfixes)
-    return;
+  texture = meta_cursor_tracker_get_sprite (recorder->cursor_tracker);
+  width = cogl_texture_get_width (texture);
+  height = cogl_texture_get_height (texture);
+  stride = 4 * width;
+  data = g_new (guint8, stride * height);
+  cogl_texture_get_data (texture, CLUTTER_CAIRO_FORMAT_ARGB32, stride, data);
 
-  cursor_image = XFixesGetCursorImage (clutter_x11_get_default_display ());
-  if (!cursor_image)
-    return;
-
-  recorder->cursor_hot_x = cursor_image->xhot;
-  recorder->cursor_hot_y = cursor_image->yhot;
-
-  recorder->cursor_image = cairo_image_surface_create (CAIRO_FORMAT_ARGB32,
-                                                       cursor_image->width,
-                                                       cursor_image->height);
-
-  /* The pixel data (in typical Xlib breakage) is longs even on
-   * 64-bit platforms, so we have to data-convert there. For simplicity,
-   * just do it always
-   */
-  data = cairo_image_surface_get_data (recorder->cursor_image);
-  stride = cairo_image_surface_get_stride (recorder->cursor_image);
-  for (i = 0; i < cursor_image->height; i++)
-    for (j = 0; j < cursor_image->width; j++)
-      *(guint32 *)(data + i * stride + 4 * j) = cursor_image->pixels[i * cursor_image->width + j];
-
-  cairo_surface_mark_dirty (recorder->cursor_image);
-
-  XFree (cursor_image);
+  /* FIXME: cairo-gl? */
+  recorder->cursor_image = cairo_image_surface_create_for_data (data,
+                                                                CAIRO_FORMAT_ARGB32,
+                                                                width, height,
+                                                                stride);
+  recorder->cursor_memory = data;
 }
 
 /* Overlay the cursor image on the frame. We draw the cursor image
@@ -475,56 +395,6 @@ recorder_draw_cursor (ShellRecorder *recorder,
   gst_buffer_unmap (buffer, &info);
 }
 
-/* Draw an overlay indicating how much of the target memory is used
- * for buffering frames.
- */
-static void
-recorder_draw_buffer_meter (ShellRecorder *recorder)
-{
-  int fill_level;
-  GdkRectangle primary_monitor;
-  float rects[16];
-
-  gdk_screen_get_monitor_workarea (recorder->gdk_screen,
-                                   gdk_screen_get_primary_monitor (recorder->gdk_screen),
-                                   &primary_monitor);
-
-  recorder_update_memory_used (recorder, FALSE);
-
-  /* As the buffer gets more full, we go from green, to yellow, to red */
-  if (recorder->memory_used > (recorder->memory_target * 3) / 4)
-    cogl_set_source_color4f (1, 0, 0, 1);
-  else if (recorder->memory_used > recorder->memory_target / 2)
-    cogl_set_source_color4f (1, 1, 0, 1);
-  else
-    cogl_set_source_color4f (0, 1, 0, 1);
-
-  fill_level = MIN (60, (recorder->memory_used * 60) / recorder->memory_target);
-
-  /* A hollow rectangle filled from the left to fill_level */
-  rects[0] = primary_monitor.x + primary_monitor.width - 64;
-  rects[1] = primary_monitor.y + primary_monitor.height - 10;
-  rects[2] = primary_monitor.x + primary_monitor.width - 2;
-  rects[3] = primary_monitor.y + primary_monitor.height - 9;
-
-  rects[4] = primary_monitor.x + primary_monitor.width - 64;
-  rects[5] = primary_monitor.y + primary_monitor.height - 9;
-  rects[6] = primary_monitor.x + primary_monitor.width - (63 - fill_level);
-  rects[7] = primary_monitor.y + primary_monitor.height - 3;
-
-  rects[8] = primary_monitor.x + primary_monitor.width - 3;
-  rects[9] = primary_monitor.y + primary_monitor.height - 9;
-  rects[10] = primary_monitor.x + primary_monitor.width - 2;
-  rects[11] = primary_monitor.y + primary_monitor.height - 3;
-
-  rects[12] = primary_monitor.x + primary_monitor.width - 64;
-  rects[13] = primary_monitor.y + primary_monitor.height - 3;
-  rects[14] = primary_monitor.x + primary_monitor.width - 2;
-  rects[15] = primary_monitor.y + primary_monitor.height - 2;
-
-  cogl_rectangles (rects, 4);
-}
-
 /* We want to time-stamp each frame based on the actual time it was
  * recorded. We probably should use the pipeline clock rather than
  * gettimeofday(): that would be needed to get sync'ed audio correct.
@@ -608,22 +478,7 @@ recorder_on_stage_paint (ClutterActor  *actor,
                          ShellRecorder *recorder)
 {
   if (recorder->state == RECORDER_STATE_RECORDING)
-    {
-      GdkRectangle primary_monitor;
-
-      gdk_screen_get_monitor_workarea (recorder->gdk_screen,
-                                       gdk_screen_get_primary_monitor (recorder->gdk_screen),
-                                       &primary_monitor);
-      if (!recorder->only_paint)
-        recorder_record_frame (recorder);
-
-      cogl_set_source_texture (recorder->recording_icon);
-      cogl_rectangle (primary_monitor.x + primary_monitor.width - 32, primary_monitor.y + primary_monitor.height - 42,
-                      primary_monitor.x + primary_monitor.width,      primary_monitor.y + primary_monitor.height - 10);
-    }
-
-  if (recorder->state == RECORDER_STATE_RECORDING || recorder->memory_used != 0)
-    recorder_draw_buffer_meter (recorder);
+    recorder_record_frame (recorder);
 }
 
 static void
@@ -682,6 +537,24 @@ recorder_queue_redraw (ShellRecorder *recorder)
                                              recorder_idle_redraw, recorder, NULL);
 }
 
+static void
+on_cursor_changed (MetaCursorTracker *tracker,
+                   ShellRecorder     *recorder)
+{
+  if (recorder->cursor_image)
+    {
+      cairo_surface_destroy (recorder->cursor_image);
+      recorder->cursor_image = NULL;
+    }
+  if (recorder->cursor_memory)
+    {
+      g_free (recorder->cursor_memory);
+      recorder->cursor_memory = NULL;
+    }
+
+  recorder_queue_redraw (recorder);
+}
+
 /* We use an event filter on the stage to get the XFixesCursorNotifyEvent
  * and also to track cursor position (when the cursor is over the stage's
  * input area); tracking cursor position here rather than with ClutterEvent
@@ -703,23 +576,8 @@ recorder_event_filter (XEvent        *xev,
       xev->xcookie.extension == recorder->xinput_opcode)
     input_event = (XIEvent *) xev->xcookie.data;
 
-  if (xev->xany.type == recorder->xfixes_event_base + XFixesCursorNotify)
-    {
-      XFixesCursorNotifyEvent *notify_event = (XFixesCursorNotifyEvent *)xev;
-
-      if (notify_event->subtype == XFixesDisplayCursorNotify)
-        {
-          if (recorder->cursor_image)
-            {
-              cairo_surface_destroy (recorder->cursor_image);
-              recorder->cursor_image = NULL;
-            }
-
-          recorder_queue_redraw (recorder);
-        }
-    }
-  else if (input_event != NULL &&
-           input_event->evtype == XI_Motion)
+  if (input_event != NULL &&
+      input_event->evtype == XI_Motion)
     {
       XIDeviceEvent *device_event = (XIDeviceEvent *) input_event;
       if (device_event->deviceid == VIRTUAL_CORE_POINTER_ID)
@@ -943,14 +801,6 @@ recorder_set_stage (ShellRecorder *recorder,
 
       recorder_update_size (recorder);
 
-      recorder->have_xfixes = XFixesQueryExtension (clutter_x11_get_default_display (),
-                                                    &recorder->xfixes_event_base,
-                                                    &error_base);
-      if (recorder->have_xfixes)
-        XFixesSelectCursorInput (clutter_x11_get_default_display (),
-                                   clutter_x11_get_stage_window (stage),
-                                 XFixesDisplayCursorNotifyMask);
-
       if (XQueryExtension (clutter_x11_get_default_display (),
                            "XInputExtension",
                            &recorder->xinput_opcode,
@@ -979,6 +829,22 @@ recorder_set_stage (ShellRecorder *recorder,
     }
 }
 
+static void
+recorder_set_screen (ShellRecorder *recorder,
+                     MetaScreen    *screen)
+{
+  MetaCursorTracker *tracker;
+
+  tracker = meta_cursor_tracker_get_for_screen (screen);
+
+  if (tracker == recorder->cursor_tracker)
+    return;
+
+  recorder->cursor_tracker = tracker;
+  g_signal_connect_object (tracker, "cursor-changed",
+                           G_CALLBACK (on_cursor_changed), recorder, 0);
+}
+
 static void
 recorder_set_framerate (ShellRecorder *recorder,
                         int framerate)
@@ -1054,6 +920,9 @@ shell_recorder_set_property (GObject      *object,
 
   switch (prop_id)
     {
+    case PROP_SCREEN:
+      recorder_set_screen (recorder, g_value_get_object (value));
+      break;
     case PROP_STAGE:
       recorder_set_stage (recorder, g_value_get_object (value));
       break;
@@ -1115,6 +984,14 @@ shell_recorder_class_init (ShellRecorderClass *klass)
   gobject_class->get_property = shell_recorder_get_property;
   gobject_class->set_property = shell_recorder_set_property;
 
+  g_object_class_install_property (gobject_class,
+                                   PROP_SCREEN,
+                                   g_param_spec_object ("screen",
+                                                        "Screen",
+                                                        "Screen to record",
+                                                        META_TYPE_SCREEN,
+                                                        G_PARAM_WRITABLE));
+
   g_object_class_install_property (gobject_class,
                                    PROP_STAGE,
                                    g_param_spec_object ("stage",
